/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.authentication.http;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionActivationListener;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;
import javax.servlet.http.HttpSessionEvent;

import net.sf.jguard.authentication.callbacks.jee.HttpCallbackHandler;
import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.principals.UserPrincipal;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.AuthenticationException;
import net.sf.jguard.ext.authentication.JGuardLoginException;
import net.sf.jguard.ext.authentication.SubjectTemplate;
import net.sf.jguard.ext.authentication.SubjectUtils;
import net.sf.jguard.ext.authentication.manager.AuthenticationManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;



/**
 * Authentication utility class stored on the user's session.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * this class was inspired by the article on jaas published at <a href="http://www.mooreds.com/jaas.html">this</a> address.
 *
 */
public class HttpAuthenticationUtils implements HttpSessionActivationListener,HttpSessionBindingListener{
	private static final Log logger = LogFactory.getLog(HttpAuthenticationUtils.class);
	//prevents if the user is already gone
    private boolean loggedOut = false;
    private LoginContext loginContext;
    private Subject subject = null;
    private HttpSession session = null;

    /**
     * encapsulate JAAS login lifecycle.
     * @param request
     * @param response
     * @param afterRegistration <code>true</code> if user registration was made just before; <code>false</code> otherwise.
     * @throws LoginException
     */
     private  void useLoginContext(HttpServletRequest request,HttpServletResponse response,boolean afterRegistration) throws LoginException {
    	session = request.getSession();

    	String applicationName = (String)session.getServletContext().getAttribute(CoreConstants.APPLICATION_NAME);
		String scheme = (String)session.getServletContext().getAttribute(HttpConstants.AUTH_SCHEME);
        HttpCallbackHandler cbh = new HttpCallbackHandler(request,response,scheme);
        cbh.setAfterRegistration(afterRegistration);

        try{
            loginContext = new LoginContext(applicationName,cbh);

        } catch(SecurityException se) {
              logger.error("LoginContext cannot be created. "+ se.getMessage(),se);
              throw new LoginException(se.getLocalizedMessage());
        }

        try{
            loginContext.login();

        }catch(LoginException le){
            //authentication failed
            logger.error("authentication failed.LoginException "+le.getMessage(),le);
            throw le;
        }
        subject = loginContext.getSubject();

        if(subject != null){
        	// used in ABAC permissions
        	UserPrincipal userPrincipal = new UserPrincipal(subject);
        	subject.getPrincipals().add(userPrincipal);
        }

      }



    /**
     * retrieve the subject from the loginContext.
     * @return authenticated Subject, otherwise <strong>null</strong>.
     */
    public Subject getSubject(){
    	return subject;
    }

    /**
     * logout the user with the related LoginContext.
     */
    public  void logout() {
      if(loggedOut==false){
    	 try {
         	if(loginContext!=null){
                loginContext.logout();
                loggedOut = true;
         	}else{
         		logger.debug(" user is not logged, so we don't logout him ");
         	}
         } catch (LoginException e) {
 			logger.error(" error raised when the user logout ", e);
         }
      }
    }

    /**
     * grab the @link HttpAuthenticationUtils from the HttpServletRequest.
     * @param req
     * @return HttpAuthenticationUtils
     */
    public static HttpAuthenticationUtils getHttpAuthenticationUtils(HttpServletRequest req){
		HttpSession session = req.getSession(true);
		HttpAuthenticationUtils auth = (HttpAuthenticationUtils) session.getAttribute(HttpConstants.AUTH_UTILS);

		//auth.getSubject() can return  null if the user in session has been deleted by authenticationManager
		//so we should use
		if(auth!= null &&  auth.getSubject()==null){
			logger.debug(" subject into HttpAuthenticationUtils is null ");
			auth.logout();
			session.removeAttribute(HttpConstants.AUTH_UTILS);
			auth = null;
		}

		if(auth==null){
			auth = new HttpAuthenticationUtils();
			session.setAttribute(HttpConstants.AUTH_UTILS,auth);
		}
    	return auth;
    }

    /**
     * authenticate user and put the corresponding Subject in its session if succeed.
     * @param req
     * @param res
     * @param afterRegistration :<code>true</code> if user registration was made just before;
     * <code>false</code> otherwise.
     * @return authenticationresult: <strong>true</strong> if authentication succeed,
     * <strong>false</strong> otherwise.
     * @throws IOException
     */
	public static boolean authenticate(HttpServletRequest req, HttpServletResponse res,boolean afterRegistration) throws IOException {

        HttpAuthenticationUtils auth = getHttpAuthenticationUtils(req);
        HttpSession session = req.getSession(true);
		boolean authenticationSucceed = true;
		try {
			auth.useLoginContext(req, res,afterRegistration);
			Subject subject = auth.getSubject();
			if(logger.isDebugEnabled()){
				logger.debug("subject retrieved="+subject);
			}
		} catch (LoginException e) {
			authenticationSucceed =  false;
			String messageError = null;
			if (e instanceof JGuardLoginException) {
				JGuardLoginException loginException = (JGuardLoginException) e;
				messageError = loginException.getLocalizedMessage(req.getLocale());
			}else{
				messageError = e.getMessage();
			}
			//we store in the user' session the reason the authentication failed
			session.setAttribute(HttpConstants.LOGIN_EXCEPTION_MESSAGE,messageError);
			session.setAttribute(HttpConstants.LOGIN_EXCEPTION_CLASS,e.getClass());
		}

		return authenticationSucceed;
	}



	/**
	 * method called by the container when session is serialized.
	 * @param sessionEvent
	 */
	public void sessionWillPassivate(HttpSessionEvent sessionEvent) {
		logout();
		HttpAuthenticationUtils authUtils = (HttpAuthenticationUtils)sessionEvent.getSession().getAttribute(HttpConstants.AUTH_UTILS);
		if(authUtils!=null){
			authUtils.logout();
		}
		sessionEvent.getSession().removeAttribute(HttpConstants.AUTH_UTILS);
	}


	/**
	 * method called by container when session is deserialized.
	 * @param sessionEvent
	 */
	public void sessionDidActivate(HttpSessionEvent sessionEvent) {


	}


	/**
	 * refresh the user stored into the HttpAuthenticationUtils to reflect dynamically changes on connected users,
	 * including delete of users.
	 * @param jcredUpdatedUser
	 * @throws AuthenticationException
	 */
	public void  refreshUser(JGuardCredential jcredUpdatedUser) throws AuthenticationException{
		AuthenticationManager authenticationManager = (AuthenticationManager)session.getServletContext().getAttribute(SecurityConstants.AUTHENTICATION_MANAGER);
		SubjectTemplate st = authenticationManager.getDefaultSubjectTemplate();
		JGuardCredential  jcredOld  = SubjectUtils.getIdentityCredential(subject,st);
		if(!jcredOld.equals(jcredUpdatedUser)){
			//we don't need to refresh this user
			//because it does not match with the one updated
			return;
		}
		Collection creds = new ArrayList();
		creds.add(jcredOld);

		//we grab the updated user
		Set users = authenticationManager.findUsers(creds);
		if(users.size()>1){
			logger.error("user cannot be updated because identity credential maps to multiple users or none ");
			throw new AuthenticationException("user cannot be updated because identity credential maps to multiple users");
		}else if(users.size()==0){
			//user has been removed
			subject = null;

		}else{
			Iterator it = users.iterator();
			//we update the user
			subject = (Subject)it.next();

		}
	}



	public void valueBound(HttpSessionBindingEvent bindingEvent) {
		Collection users = (Collection)bindingEvent.getSession().getServletContext().getAttribute(HttpConstants.USERS_IN_SESSION);
		users.add(this);
	}



	public void valueUnbound(HttpSessionBindingEvent bindingEvent) {
		Collection users = (Collection)bindingEvent.getSession().getServletContext().getAttribute(HttpConstants.USERS_IN_SESSION);
		users.remove(this);
	}

	 /**
     * refresh all users which match with the identityCredential present in the Subject updated.
     * note that the identityCredential must'nt be changed.
     * @param subject subject updated
     * @param servletContext
	 * @throws AuthenticationException
     */
    public static void refreshUsers(Subject subject,ServletContext servletContext) throws AuthenticationException{

    	AuthenticationManager authenticationManager = (AuthenticationManager)servletContext.getAttribute(SecurityConstants.AUTHENTICATION_MANAGER);
    	SubjectTemplate st = authenticationManager.getDefaultSubjectTemplate();
		JGuardCredential  jcredUpdatedUser = SubjectUtils.getIdentityCredential(subject,st);
		refreshUsers(jcredUpdatedUser,servletContext);
    }


    /**
     * refresh all users which match with the identityCredential.
     * @param servletContext
     * @param identityCredential
     * @throws AuthenticationException
     */
	public static void refreshUsers(JGuardCredential identityCredential,ServletContext servletContext) throws AuthenticationException {
		Collection httpAuthUtilsColl = (Collection)servletContext.getAttribute(HttpConstants.USERS_IN_SESSION);
    	Iterator it = httpAuthUtilsColl.iterator();
		while(it.hasNext()){
			HttpAuthenticationUtils authUtils = (HttpAuthenticationUtils)it.next();
			authUtils.refreshUser(identityCredential);
		}
	}

}