/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.

http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.listeners;

import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.security.Policy;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.management.remote.rmi.RMIConnectorServer;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import net.sf.jguard.authentication.http.HttpConstants;
import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authorization.policy.MultipleAppPolicy;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.jmx.JGuardJMXAuthenticator;
import net.sf.jguard.ext.authentication.manager.AuthenticationHelper;
import net.sf.jguard.ext.authentication.manager.AuthenticationManager;
import net.sf.jguard.ext.authorization.AuthorizationHelper;
import net.sf.jguard.ext.authorization.manager.AuthorizationManager;
import net.sf.jguard.util.jee.WebappUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sun.jndi.rmi.registry.RegistryContextFactory;

/**
 * handle webapp classloader reference in the
 * JGuardPolicy classloaders repository.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:vberetti@users.sourceforge.net">Vincent Beretti</a>
 */
public class ContextListener implements ServletContextListener {

	private static final String JAVA_NAMING_FACTORY_INITIAL = "java.naming.factory.initial";

	private static final Log logger = LogFactory.getLog(ContextListener.class);

    private AuthenticationManager authenticationManager = null;
    private AuthorizationManager authorizationManager = null;

    /**
	 * method called when the webapp shutdown:
	 * this method unregister the webapp in the JGuardPolicy repository.
	 * @see javax.servlet.ServletContextListener#contextDestroyed(javax.servlet.ServletContextEvent)
	 */
	public void contextDestroyed(ServletContextEvent servletContextEvent) {
		logger.debug(" context destroyed ");
		ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
		if(Policy.getPolicy() instanceof MultipleAppPolicy){
		  MultipleAppPolicy policy = (MultipleAppPolicy) Policy.getPolicy();
          policy.unregisterPermissionProvider(contextClassLoader);
		}

		servletContextEvent.getServletContext().removeAttribute(SecurityConstants.CAPTCHA_SERVICE);
		servletContextEvent.getServletContext().removeAttribute(HttpConstants.USERS_IN_SESSION);
	}

	/**
	 * method called when the webapp start.
	 * install jGuard Configuration and Policy
	 * @see javax.servlet.ServletContextListener#contextInitialized(javax.servlet.ServletContextEvent)
	 */
	public void contextInitialized(ServletContextEvent contextEvent) {
		logger.debug("#####  initializing ContextListener ... #####");
		ServletContext context = contextEvent.getServletContext();
		context.setAttribute(HttpConstants.USERS_IN_SESSION,new ArrayList());

		String webappHomePath = WebappUtil.getWebappHomePath(context);
		String applicationName = context.getServletContextName();

		context.setAttribute(HttpConstants.WEBAPP_HOME_PATH,webappHomePath);
		context.setAttribute(CoreConstants.APPLICATION_NAME,applicationName);

		//use default authenticationConfigurationLocation unless it is explicitly defined
		String authenticationConfigurationLocation=null;
		if(context.getInitParameter(HttpConstants.AUTHENTICATION_CONFIGURATION_LOCATION)!=null){
			authenticationConfigurationLocation = "file://"+context.getRealPath(context.getInitParameter(HttpConstants.AUTHENTICATION_CONFIGURATION_LOCATION));
		}else{
			authenticationConfigurationLocation="file://"+context.getRealPath(HttpConstants.DEFAULT_AUTHENTICATION_CONFIGURATION_LOCATION);
		}

		// use default authorizationConfigurationLocation unless it is explicitly defined
		String authorizationConfigurationLocation=null;
		if(context.getInitParameter(HttpConstants.AUTHORIZATION_CONFIGURATION_LOCATION)!=null){
			authorizationConfigurationLocation = "file://"+context.getRealPath(context.getInitParameter(HttpConstants.AUTHORIZATION_CONFIGURATION_LOCATION));
		}else{
			authorizationConfigurationLocation="file://"+context.getRealPath(HttpConstants.DEFAULT_AUTHORIZATION_CONFIGURATION_LOCATION);
		}

		Map authenticationSettings = AuthenticationHelper.loadConfiguration(authenticationConfigurationLocation,webappHomePath);
		Map authorizationSettings = AuthorizationHelper.loadConfiguration(authorizationConfigurationLocation,webappHomePath);

		authenticationManager = AuthenticationHelper.initAuthentication(authenticationSettings,applicationName);
		context.setAttribute(SecurityConstants.AUTHENTICATION_MANAGER,authenticationManager);

        authorizationManager = AuthorizationHelper.initAuthorization(authorizationSettings,applicationName);
        context.setAttribute(SecurityConstants.AUTHORIZATION_MANAGER,authorizationManager);

        enableJMXSecurity(context, applicationName);

		logger.debug("#####  ContextListener initialized  #####");
	}

	private void enableJMXSecurity(ServletContext context, String applicationName) {
		// JMX MBeanServer creation
        String enableJMX=(String)context.getInitParameter(HttpConstants.ENABLE_JMX);
        if(enableJMX != null && "true".equals(enableJMX)){
        	logger.info("Starting JMX Server ...");
        	//does each webapp create its own MBeanServer
        	//or do webapps share their MBeans on the same MBeanServer ?
        	MBeanServer mbs = MBeanServerFactory.createMBeanServer(applicationName);

        	String rmiRegistryHost=HttpConstants.DEFAULT_RMI_REGISTRY_HOST;
        	if(context.getInitParameter(HttpConstants.RMI_REGISTRY_HOST)!=null){
        		rmiRegistryHost=(String)context.getInitParameter(HttpConstants.RMI_REGISTRY_HOST);
        	}

        	int rmiRegistryPort=HttpConstants.DEFAULT_RMI_REGISTRY_PORT;
        	if(context.getInitParameter(HttpConstants.RMI_REGISTRY_PORT)!=null){
        		try{
        		rmiRegistryPort=Integer.parseInt((String)context.getInitParameter(HttpConstants.RMI_REGISTRY_PORT));
        		}catch(NumberFormatException e){
        			logger.warn("RmiRegistry port in web.xml is not a number. Using default.");
        		}
        	}

        	try{
        		// TODO handle custom rmiRegistryHost
    			LocateRegistry.createRegistry(rmiRegistryPort);
//        		LocateRegistry.getRegistry(rmiRegistryHost,rmiRegistryPort);

        	}catch(RemoteException e){
        		logger.error("Could not found rmiRegistry : "+e.getMessage());
        	}

        	String serviceURL = "service:jmx:rmi://localhost/jndi/rmi://"+rmiRegistryHost+":"+rmiRegistryPort+"/"+applicationName;
    		JMXServiceURL url=null;
    		try {
    			url=new JMXServiceURL(serviceURL);
    		} catch (MalformedURLException e) {
    			logger.error("MalformedURLException : "+e);
    		}
    		logger.info("JMX Server URL : "+url.toString());

    		JGuardJMXAuthenticator jmxAuthenticator=new JGuardJMXAuthenticator(applicationName,Thread.currentThread().getContextClassLoader());

    		Map options=new HashMap();
    		options.put(JMXConnectorServer.AUTHENTICATOR,jmxAuthenticator);
    		options.put(JAVA_NAMING_FACTORY_INITIAL,RegistryContextFactory.class.getName());
    		options.put(RMIConnectorServer.JNDI_REBIND_ATTRIBUTE,"true");
    		try {
    			JMXConnectorServer connectorServer=	JMXConnectorServerFactory.newJMXConnectorServer(url,options,mbs);
    			connectorServer.start();
    		} catch (IOException e) {
    			logger.error("IOException : "+e);
    		}

        }
	}

}
